@@ -718,10 +718,7 @@ close_async_thread (GSimpleAsyncResult *res,
   class = G_FILE_ENUMERATOR_GET_CLASS (object);
   result = class->close_fn (G_FILE_ENUMERATOR (object), cancellable, &error);
   if (!result)
-    {
-      g_simple_async_result_set_from_error (res, error);
-      g_error_free (error);
-    }
+    g_simple_async_result_take_error (res, error);
 }
 
 
