@@ -34,13 +34,12 @@
 #include "gioerror.h"
 #include "glibintl.h"
 
-#include "gioalias.h"
 
 /**
  * SECTION:gmount
  * @short_description: Mount management
  * @include: gio/gio.h
- * @see also: GVolume, GUnixMount
+ * @see_also: GVolume, GUnixMount
  *
  * The #GMount interface represents user-visible mounts. Note, when 
  * porting from GnomeVFS, #GMount is the moral equivalent of #GnomeVFSVolume.
@@ -123,7 +122,7 @@ g_mount_default_init (GMountInterface *iface)
  * 
  * Gets the root directory on @mount.
  * 
- * Returns: a #GFile. 
+ * Returns: (transfer full): a #GFile. 
  *      The returned object should be unreffed with 
  *      g_object_unref() when no longer needed.
  **/
@@ -147,7 +146,7 @@ g_mount_get_root (GMount *mount)
  * @mount is a path that reflects the main entry point for the user (e.g.
  * the home directory, or the root of the volume).
  *
- * Returns: a #GFile.
+ * Returns: (transfer full): a #GFile.
  *      The returned object should be unreffed with
  *      g_object_unref() when no longer needed.
  **/
@@ -198,7 +197,7 @@ g_mount_get_name (GMount *mount)
  * 
  * Gets the icon for @mount.
  * 
- * Returns: a #GIcon.
+ * Returns: (transfer full): a #GIcon.
  *      The returned object should be unreffed with 
  *      g_object_unref() when no longer needed.
  **/
@@ -245,7 +244,7 @@ g_mount_get_uuid (GMount *mount)
  * 
  * Gets the volume for the @mount.
  * 
- * Returns: a #GVolume or %NULL if @mount is not associated with a volume.
+ * Returns: (transfer full): a #GVolume or %NULL if @mount is not associated with a volume.
  *      The returned object should be unreffed with 
  *      g_object_unref() when no longer needed.
  **/
@@ -270,7 +269,7 @@ g_mount_get_volume (GMount *mount)
  * This is a convenience method for getting the #GVolume and then
  * using that object to get the #GDrive.
  * 
- * Returns: a #GDrive or %NULL if @mount is not associated with a volume or a drive.
+ * Returns: (transfer full): a #GDrive or %NULL if @mount is not associated with a volume or a drive.
  *      The returned object should be unreffed with 
  *      g_object_unref() when no longer needed.
  **/
@@ -801,7 +800,7 @@ g_mount_guess_content_type (GMount              *mount,
  * %G_IO_ERROR_NOT_SUPPORTED if the mount does not support content 
  * guessing.
  * 
- * Returns: a %NULL-terminated array of content types or %NULL on error. 
+ * Returns: (transfer full) (element-type utf8): a %NULL-terminated array of content types or %NULL on error. 
  *     Caller should free this array with g_strfreev() when done with it.
  *
  * Since: 2.18
@@ -813,14 +812,14 @@ g_mount_guess_content_type_finish (GMount        *mount,
 {
   GMountIface *iface;
 
-  g_return_val_if_fail (G_IS_MOUNT (mount), FALSE);
-  g_return_val_if_fail (G_IS_ASYNC_RESULT (result), FALSE);
+  g_return_val_if_fail (G_IS_MOUNT (mount), NULL);
+  g_return_val_if_fail (G_IS_ASYNC_RESULT (result), NULL);
 
   if (G_IS_SIMPLE_ASYNC_RESULT (result))
     {
       GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (result);
       if (g_simple_async_result_propagate_error (simple, error))
-        return FALSE;
+        return NULL;
     }
   
   iface = G_MOUNT_GET_IFACE (mount);
@@ -845,7 +844,7 @@ g_mount_guess_content_type_finish (GMount        *mount,
  * This is an synchronous operation and as such may block doing IO;
  * see g_mount_guess_content_type() for the asynchronous version.
  *
- * Returns: a %NULL-terminated array of content types or %NULL on error.
+ * Returns: (transfer full) (element-type utf8): a %NULL-terminated array of content types or %NULL on error.
  *     Caller should free this array with g_strfreev() when done with it.
  *
  * Since: 2.18
@@ -1010,6 +1009,3 @@ g_mount_unshadow (GMount *mount)
     g_warning ("Shadow ref count on GMount is negative");
   G_UNLOCK (priv_lock);
 }
-
-#define __G_MOUNT_C__
-#include "gioaliasdef.c"
