@@ -372,7 +372,7 @@ g_simple_async_result_new_from_error (GObject             *source_object,
  *
  * Returns: a #GSimpleAsyncResult
  *
- * Since: 2.26
+ * Since: 2.28
  **/
 GSimpleAsyncResult *
 g_simple_async_result_new_take_error (GObject             *source_object,
@@ -639,7 +639,7 @@ g_simple_async_result_set_from_error (GSimpleAsyncResult *simple,
  * Sets the result from @error, and takes over the caller's ownership
  * of @error, so the caller does not need to free it any more.
  *
- * Since: 2.26
+ * Since: 2.28
  **/
 void
 g_simple_async_result_take_error (GSimpleAsyncResult *simple,
@@ -987,7 +987,7 @@ void
 g_simple_async_report_gerror_in_idle (GObject *object,
 				      GAsyncReadyCallback callback,
 				      gpointer user_data,
-				      GError *error)
+				      const GError *error)
 {
   GSimpleAsyncResult *simple;
  
@@ -1001,3 +1001,35 @@ g_simple_async_report_gerror_in_idle (GObject *object,
   g_simple_async_result_complete_in_idle (simple);
   g_object_unref (simple);
 }
+
+/**
+ * g_simple_async_report_take_gerror_in_idle:
+ * @object: a #GObject.
+ * @callback: a #GAsyncReadyCallback.
+ * @user_data: user data passed to @callback.
+ * @error: the #GError to report
+ *
+ * Reports an error in an idle function. Similar to
+ * g_simple_async_report_gerror_in_idle(), but takes over the caller's
+ * ownership of @error, so the caller does not have to free it any more.
+ *
+ * Since: 2.28
+ **/
+void
+g_simple_async_report_take_gerror_in_idle (GObject *object,
+                                           GAsyncReadyCallback callback,
+                                           gpointer user_data,
+                                           GError *error)
+{
+  GSimpleAsyncResult *simple;
+
+  g_return_if_fail (G_IS_OBJECT (object));
+  g_return_if_fail (error != NULL);
+
+  simple = g_simple_async_result_new_take_error (object,
+                                                 callback,
+                                                 user_data,
+                                                 error);
+  g_simple_async_result_complete_in_idle (simple);
+  g_object_unref (simple);
+}
