@@ -50,7 +50,6 @@
 #include <string.h>
 #endif
 
-#include "gioalias.h"
 
 G_DEFINE_TYPE_WITH_CODE (GUnixConnection, g_unix_connection,
 			 G_TYPE_SOCKET_CONNECTION,
@@ -64,8 +63,8 @@ G_DEFINE_TYPE_WITH_CODE (GUnixConnection, g_unix_connection,
  * g_unix_connection_send_fd:
  * @connection: a #GUnixConnection
  * @fd: a file descriptor
- * @cancellable: optional #GCancellable object, %NULL to ignore.
- * @error: #GError for error reporting, or %NULL to ignore.
+ * @cancellable: (allow-none): optional #GCancellable object, %NULL to ignore.
+ * @error: (allow-none): #GError for error reporting, or %NULL to ignore.
  *
  * Passes a file descriptor to the recieving side of the
  * connection. The recieving end has to call g_unix_connection_receive_fd()
@@ -118,8 +117,8 @@ g_unix_connection_send_fd (GUnixConnection  *connection,
 /**
  * g_unix_connection_receive_fd:
  * @connection: a #GUnixConnection
- * @cancellable: optional #GCancellable object, %NULL to ignore
- * @error: #GError for error reporting, or %NULL to ignore
+ * @cancellable: (allow-none): optional #GCancellable object, %NULL to ignore
+ * @error: (allow-none): #GError for error reporting, or %NULL to ignore
  *
  * Receives a file descriptor from the sending end of the connection.
  * The sending end has to call g_unix_connection_send_fd() for this
@@ -132,7 +131,7 @@ g_unix_connection_send_fd (GUnixConnection  *connection,
  * Returns: a file descriptor on success, -1 on error.
  *
  * Since: 2.22
- */
+ **/
 gint
 g_unix_connection_receive_fd (GUnixConnection  *connection,
                               GCancellable     *cancellable,
@@ -317,7 +316,8 @@ gboolean                g_unix_connection_create_pair                   (GUnixCo
  * byte to the stream, as this is required for credentials passing to
  * work on some implementations.
  *
- * Note that this function only works on Linux, currently.
+ * Other ways to exchange credentials with a foreign peer includes the
+ * #GUnixCredentialsMessage type and g_socket_get_credentials() function.
  *
  * Returns: %TRUE on success, %FALSE if @error is set.
  *
@@ -383,7 +383,10 @@ g_unix_connection_send_credentials (GUnixConnection      *connection,
  * single byte from the stream, as this is required for credentials
  * passing to work on some implementations.
  *
- * Returns: Received credentials on success (free with
+ * Other ways to exchange credentials with a foreign peer includes the
+ * #GUnixCredentialsMessage type and g_socket_get_credentials() function.
+ *
+ * Returns: (transfer full): Received credentials on success (free with
  * g_object_unref()), %NULL if @error is set.
  *
  * Since: 2.26
@@ -548,6 +551,3 @@ g_unix_connection_receive_credentials (GUnixConnection      *connection,
   g_object_unref (socket);
   return ret;
 }
-
-#define __G_UNIX_CONNECTION_C__
-#include "gioaliasdef.c"
