@@ -27,7 +27,12 @@
 #include <stdlib.h>
 #include <errno.h>
 
-#include "glib.h"
+#include "gmarkup.h"
+
+#include "galloca.h"
+#include "gstrfuncs.h"
+#include "gstring.h"
+#include "gtestutils.h"
 #include "glibintl.h"
 
 GQuark
@@ -1962,7 +1967,7 @@ g_markup_parse_context_get_user_data (GMarkupParseContext *context)
  **/
 void
 g_markup_parse_context_push (GMarkupParseContext *context,
-                             GMarkupParser       *parser,
+                             const GMarkupParser *parser,
                              gpointer             user_data)
 {
   GMarkupRecursionTracker *tracker;
